﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;

namespace SimpleSerialVixen
{
    public partial class SerialSetup : Form
    {
        string _portName;
        int _baudRate;
        Parity _parity;
        int _dataBits;
        StopBits _stopBits;
        Handshake _handshake;

        public SerialSetup()
        {
            InitializeComponent();

            SetupPortNameCombo();
            SetupBuadRateCombo();
            SetupParityCombo();
            SetupDataBitCombo();
            SetupStopBitCombo();
            SetupHandshakeCombo();

        }

        private void SetupHandshakeCombo()
        {
            List<KeyValuePair<string, Handshake>> dataSource = new List<KeyValuePair<string, Handshake>>();

            string[] labels = Enum.GetNames(typeof(Handshake));
            foreach (string label in labels)
            {
                dataSource.Add(new KeyValuePair<string, Handshake>(label, (Handshake)Enum.Parse(typeof(Handshake), label)));
            }

            cboHandshake.DataSource = dataSource;
            cboHandshake.ValueMember = "Value";
            cboHandshake.DisplayMember = "Key";
        }

        private void SetupStopBitCombo()
        {
            List<KeyValuePair<string, StopBits>> dataSource = new List<KeyValuePair<string, StopBits>>();

            string[] labels = Enum.GetNames(typeof(StopBits));
            foreach (string label in labels)
            {
                dataSource.Add(new KeyValuePair<string, StopBits>(label, (StopBits)Enum.Parse(typeof(StopBits), label)));
            }

            cboStopBits.DataSource = dataSource;
            cboStopBits.ValueMember = "Value";
            cboStopBits.DisplayMember = "Key";
        }

        private void SetupDataBitCombo()
        {
            List<KeyValuePair<string, int>> dataSource = new List<KeyValuePair<string, int>>();

            dataSource.Add(new KeyValuePair<string, int>("1", 1));
            dataSource.Add(new KeyValuePair<string, int>("2", 2));
            dataSource.Add(new KeyValuePair<string, int>("3", 3));
            dataSource.Add(new KeyValuePair<string, int>("4", 4));
            dataSource.Add(new KeyValuePair<string, int>("5", 5));
            dataSource.Add(new KeyValuePair<string, int>("6", 6));
            dataSource.Add(new KeyValuePair<string, int>("7", 7));
            dataSource.Add(new KeyValuePair<string, int>("8", 8));
            dataSource.Add(new KeyValuePair<string, int>("9", 9));

            cboDataBits.DataSource = dataSource;
            cboDataBits.ValueMember = "Value";
            cboDataBits.DisplayMember = "Key";
        }

        private void SetupParityCombo()
        {
            List<KeyValuePair<string, Parity>> dataSource = new List<KeyValuePair<string, Parity>>();

            string [] parityLabels = Enum.GetNames(typeof(Parity));
            foreach (string parityLabel in parityLabels)
            {
                dataSource.Add(new KeyValuePair<string, Parity>(parityLabel, (Parity)Enum.Parse(typeof(Parity), parityLabel)));
            }

            cboParity.DataSource = dataSource;
            cboParity.ValueMember = "Value";
            cboParity.DisplayMember = "Key";
        }

        private void SetupBuadRateCombo()
        {
            List<KeyValuePair<string, int>> dataSource = new List<KeyValuePair<string, int>>();

            dataSource.Add(new KeyValuePair<string, int>("1200", 1200));
            dataSource.Add(new KeyValuePair<string, int>("2400", 2400));
            dataSource.Add(new KeyValuePair<string, int>("4800", 4800));
            dataSource.Add( new KeyValuePair<string,int>("9600", 9600));
            dataSource.Add(new KeyValuePair<string, int>("19.2k", 19200));
            dataSource.Add(new KeyValuePair<string, int>("38.4k", 38400));
            dataSource.Add(new KeyValuePair<string, int>("115.2k", 115200));

            cboBaudRate.DataSource = dataSource;
            cboBaudRate.ValueMember = "Value";
            cboBaudRate.DisplayMember = "Key";
        }

        private void SetupPortNameCombo()
        {
            List<KeyValuePair<string, string>> dataSource = new List<KeyValuePair<string, string>>();
            string[] ports = SerialPort.GetPortNames();
            foreach (string port in ports)
            {
                dataSource.Add(new KeyValuePair<string, string>(port, port));
            }

            cboPortName.DataSource = dataSource;
            cboPortName.ValueMember = "Value";
            cboPortName.DisplayMember = "Key";
        }

        public int PortIndex
        {
            get
            {
                return cboPortName.SelectedIndex;
            }
        }

        public string PortName
        {
            get
            {
                return cboPortName.SelectedValue == null ? string.Empty : cboPortName.SelectedValue as string;
            }

            set
            {
                _portName = value;
            }
        }

        public int BaudRate
        {
            get
            {
                return cboBaudRate.SelectedValue == null ? 0 : (int)cboBaudRate.SelectedValue;
            }

            set
            {
                _baudRate = value;
            }
        }

        public Parity Parity
        {
            get
            {
                return cboParity.SelectedValue == null ? System.IO.Ports.Parity.None : (Parity)cboParity.SelectedValue;
            }

            set
            {
                _parity = value;
            }
        }

        public int DataBits
        {
            get
            {
                return cboDataBits.SelectedValue == null ? 0 : (int)cboDataBits.SelectedValue;
            }

            set
            {
                _dataBits = value;
            }
        }

        public StopBits StopBits
        {
            get
            {
                return cboStopBits.SelectedValue == null ? System.IO.Ports.StopBits.None : (StopBits)cboStopBits.SelectedValue;
            }

            set
            {
                _stopBits = value;
            }
        }

        public Handshake Handshake
        {
            get
            {
                return cboHandshake.SelectedValue == null ? System.IO.Ports.Handshake.None : (Handshake)cboHandshake.SelectedValue;
            }

            set
            {
                _handshake = value;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            Close();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void SerialSetup_Activated(object sender, EventArgs e)
        {
            try
            {
                cboBaudRate.SelectedValue = _baudRate;
                cboPortName.SelectedValue = _portName;
                cboParity.SelectedValue = _parity;
                cboDataBits.SelectedValue = _dataBits;
                cboStopBits.SelectedValue = _stopBits;
                cboHandshake.SelectedValue = _handshake;
            }
            catch
            { }

        }
    }
}
