using System;
using System.Collections.Generic;
using System.Text;
using Vixen;
using System.Xml;
using System.IO.Ports;
using System.Windows.Forms;

namespace SimpleSerialVixen
{
    public class SimpleSerial : PlugIn
    {
        
        public SimpleSerial()
        {
            this._selectedPort = null;
            this._portIndex = 0;
            this._dialogList = new List<Form>();
            this._serialPort = new SerialPort();
            this._serialPort.Handshake = Handshake.None;
            this._serialPort.Encoding = Encoding.UTF8;
        }

        private int IndexOfPort(short port)
        {
            switch (port)
            {
                case 0x2e8:
                    return 2;

                case 0x3e8:
                    return 1;

                case 0x3f8:
                    return 0;
            }
            return -1;
        }

        private void SetPort(SerialPortParms portParams)
        {
            this._serialPort.PortName = portParams.PortName;
            this._serialPort.BaudRate = portParams.Baud;
            this._serialPort.Parity = portParams.Parity;
            this._serialPort.DataBits = portParams.DataBits;
            this._serialPort.StopBits = portParams.StopBits;
            this._portIndex = Convert.ToInt32((int)(this._serialPort.PortName[3] - '1'));
        }

        public override string ToString()
        {
            return this.Name;
        }
        #region PlugIn Members

        public string Author
        {
            get { return "SRM"; }
        }

        public string Description
        {
            get { return "A simple rs232 serial interface."; }
        }

        public void Event(byte[] channelValues)
        {
            // left and right channels are mutually exclusive
            // first one wins (can't have both on simultaneously
            if (channelValues[0] == 0 && channelValues[1] == 0)
            {
                // center
                writeCenterHorizontalCommand();
                
            }
            else if (channelValues[0] > 0)
            {
                writeLeftCommand(channelValues[0]);
            }
            else
            {
                writeRightCommand(channelValues[1]);
            }

            // ditto for up/down
            if (channelValues[2] == 0 && channelValues[3] == 0)
            {
                // center
                writeCenterVerticalCommand();

            }
            else if (channelValues[2] > 0)
            {
                writeUpCommand(channelValues[2]);
            }
            else
            {
                writeDownCommand(channelValues[3]);
            }
        }

        private void writeDownCommand(byte value)
        {
            _serialPort.Write("[D");
            _serialPort.Write(new byte[] { value }, 0, 1);
        }

        private void writeUpCommand(byte value)
        {
            _serialPort.Write("[U");
            _serialPort.Write(new byte[] { value }, 0, 1);
        }

        private void writeCenterVerticalCommand()
        {
            _serialPort.Write("[U");
            _serialPort.Write(new byte[] { 0 }, 0, 1);
        }

        private void writeCenterHorizontalCommand()
        {
            _serialPort.Write("[R");
            _serialPort.Write(new byte[] { 0 }, 0, 1);
        }

        private void writeRightCommand(byte value)
        {
            _serialPort.Write("[R");
            _serialPort.Write(new byte[] { value }, 0 , 1);
        }

        private void writeLeftCommand(byte value)
        {
            _serialPort.Write("[L");
            _serialPort.Write(new byte[] { value }, 0, 1);
        }

        public HardwareMap[] HardwareMap
        {
            get
            {
                return new HardwareMap[] { new HardwareMap(PortType.Serial, this.IndexOfPort(this._selectedPort.Port)) };
            }}

        public void Initialize(Channel[] channels, SetupData setupData, System.Xml.XmlNode setupNode)
        {
            this._setupData = setupData;
            this._setupNode = setupNode;
            this._selectedPort = new SerialPortParms(this._setupData.GetString(this._setupNode, "name", "COM1"), Convert.ToInt16(this._setupData.GetString(this._setupNode, "port", "3f8"), 0x10), this._setupData.GetInteger(this._setupNode, "baud", 0x4b00), this._setupData.GetInteger(this._setupNode, "data", 8), (StopBits)Enum.Parse(typeof(StopBits), this._setupData.GetString(this._setupNode, "stop", StopBits.One.ToString())), (Parity)Enum.Parse(typeof(Parity), this._setupData.GetString(this._setupNode, "parity", Parity.None.ToString())));
            this.SetPort(this._selectedPort);
        }

        public string Name
        {
            get { return "Simple Serial"; }
        }

        public void Setup()
        {
            SerialSetup dialog = new SerialSetup();

            dialog.PortName = _serialPort.PortName;
            dialog.BaudRate = _serialPort.BaudRate;
            dialog.StopBits = _serialPort.StopBits;
            dialog.Handshake = _serialPort.Handshake;
            dialog.DataBits = _serialPort.DataBits;

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _selectedPort.PortName = dialog.PortName;
                _selectedPort.Baud = dialog.BaudRate;
                _selectedPort.DataBits = dialog.DataBits;
                _selectedPort.StopBits = dialog.StopBits;
                _selectedPort.Port =  _ports[ Math.Max(dialog.PortIndex, 2) ];

                this._setupData.SetString(this._setupNode, "name", this._selectedPort.PortName);
                this._setupData.SetString(this._setupNode, "port", this._selectedPort.Port.ToString("x"));
                this._setupData.SetInteger(this._setupNode, "baud", this._selectedPort.Baud);
                this._setupData.SetString(this._setupNode, "parity", this._selectedPort.Parity.ToString());
                this._setupData.SetInteger(this._setupNode, "data", this._selectedPort.DataBits);
                this._setupData.SetString(this._setupNode, "stop", this._selectedPort.StopBits.ToString());
                this.SetPort(this._selectedPort);
            }

        }

        public void Shutdown()
        {
            this._serialPort.Close();
            this._dialogList.Clear();
        }

        public List<System.Windows.Forms.Form> Startup()
        {
            this._serialPort.Open();
            return this._dialogList;
        }

        #endregion

        private SetupData _setupData;
        private XmlNode _setupNode;
        private SerialPortParms _selectedPort;
        private SerialPort _serialPort;
        private int _portIndex;
        private List<System.Windows.Forms.Form> _dialogList;
        private short[] _ports = new short[] {0x3f8, 0x3e8, 0x2e8 };
    }
}
